@@ -15,6 +15,7 @@
 
 package org.eclipse.app4mc.amalthea.model.util;
 
+import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -36,7 +37,6 @@ import org.eclipse.app4mc.amalthea.model.CustomStimulus;
 import org.eclipse.app4mc.amalthea.model.EventStimulus;
 import org.eclipse.app4mc.amalthea.model.ExecutionNeed;
 import org.eclipse.app4mc.amalthea.model.Frequency;
-import org.eclipse.app4mc.amalthea.model.HWModel;
 import org.eclipse.app4mc.amalthea.model.HwFeature;
 import org.eclipse.app4mc.amalthea.model.HwFeatureCategory;
 import org.eclipse.app4mc.amalthea.model.IDiscreteValueDeviation;
@@ -59,6 +59,7 @@ import org.eclipse.app4mc.amalthea.model.Time;
 import org.eclipse.app4mc.amalthea.model.TimeUnit;
 import org.eclipse.app4mc.amalthea.model.VariableRateStimulus;
 import org.eclipse.emf.common.util.EMap;
+import org.eclipse.emf.ecore.EObject;
 
 public class RuntimeUtil {
 
@@ -88,20 +89,20 @@ public class RuntimeUtil {
 	 * @return execution time
 	 */
 	public static Time getExecutionTimeForProcess(Process process, EMap<ModeLabel, String> modes, TimeType executionCase) {
-		Time result = FactoryUtil.createTime();
-
-		Set<ProcessingUnit> processingUnits = DeploymentUtil.getAssignedCoreForProcess(process, (Amalthea) process.eContainer().eContainer());
-		List<Runnable> runnables = SoftwareUtil.getRunnableList(process, modes);
-		if (processingUnits.size() == 1) {
-			ProcessingUnit processingUnit = processingUnits.iterator().next(); 
-			for (Runnable runnable : runnables ) {
-				result = result.add(getExecutionTimeForRunnable(runnable, processingUnit, modes, executionCase));
-			}
+		Time dummy = FactoryUtil.createTime();
+		if (process == null) return dummy;
+		
+		Amalthea model = getContainingModel(process);
+		if (model == null) return dummy;
+		
+		Set<ProcessingUnit> puSet = DeploymentUtil.getAssignedCoreForProcess(process);
+		if (puSet.size() != 1) {
+			System.out.println("Mapping unclear. Process mapped to " + puSet.size() + " Cores - Use PU specific method");
+			return dummy;
 		}
-		else {
-			System.out.println("Mapping unclear. Process mapped to " + processingUnits.size() + " Cores - Use PU specific method");
-		}
-		return result;
+		
+		ProcessingUnit processingUnit = puSet.iterator().next(); 
+		return getExecutionTimeForProcess(process, processingUnit, modes, executionCase);
 	}
 
 	/**
@@ -115,9 +116,9 @@ public class RuntimeUtil {
 	 */
 	public static Time getExecutionTimeForProcess(Process process, ProcessingUnit processingUnit, EMap<ModeLabel, String> modes, TimeType executionCase) {
 		Time result = FactoryUtil.createTime();
-
-		List<Runnable> runnables = SoftwareUtil.getRunnableList(process, modes);
-		for (Runnable runnable : runnables ) {
+		if (process == null || processingUnit == null) return result;
+		
+		for (Runnable runnable : SoftwareUtil.getRunnableList(process, modes) ) {
 			result = result.add(getExecutionTimeForRunnable(runnable, processingUnit, modes, executionCase));
 		}
 		return result;
@@ -159,7 +160,7 @@ public class RuntimeUtil {
 	 * Computes time for ticks on a given processing unit
 	 */
 	public static Time getExecutionTimeForTicks(Ticks ticks, ProcessingUnit processingUnit, TimeType executionCase) {
-		Time result = FactoryUtil.createTime();
+		Time result = null;
 
 		if (ticks.getExtended().get(processingUnit.getDefinition()) != null) {
 			IDiscreteValueDeviation deviation = ticks.getExtended().get(processingUnit.getDefinition());
@@ -192,8 +193,7 @@ public class RuntimeUtil {
 			ticks = deviation.getUpperBound().doubleValue();
 			break;
 		}
-		Time result = getExecutionTimeForCycles(ticks, processingUnit.getFrequencyDomain().getDefaultValue());
-		return result;
+		return getExecutionTimeForCycles(ticks, processingUnit.getFrequencyDomain().getDefaultValue());
 	}
 
 	/**
@@ -206,16 +206,22 @@ public class RuntimeUtil {
 	 */
 	public static Time getExecutionTimeForExecutionNeeds(ExecutionNeed need, ProcessingUnit processingUnit, TimeType executionCase) {
 		Time result = FactoryUtil.createTime();
-
-		HWModel hwModel = (HWModel) processingUnit.getDefinition().eContainer();
+		if (need == null || processingUnit == null) return result;
+		
+		Amalthea model = getContainingModel(processingUnit);
+		if (model == null) return result;
+		
 		for (Entry<String, IDiscreteValueDeviation> needEntry : need.getNeeds()) {
-			Set<? extends HwFeatureCategory> hwFeatureCategory = AmaltheaIndex.getElements(hwModel, needEntry.getKey(), HwFeatureCategory.class);
-			if (hwFeatureCategory.size() == 1) {
-				result = getExecutionTimeForExecutionNeedEntry(needEntry.getValue(), hwFeatureCategory.iterator().next(), processingUnit, executionCase);
+			String categoryName = needEntry.getKey();
+			if (categoryName == null) continue;
+			
+			Set<HwFeatureCategory> categorySet = AmaltheaIndex.getElements(model, categoryName, HwFeatureCategory.class);
+			if (categorySet.size() == 1) {
+				result = getExecutionTimeForExecutionNeedEntry(needEntry.getValue(), categorySet.iterator().next(), processingUnit, executionCase);
 			}
 			else {
 				//should not happen since name is unique identifier
-				System.out.println("Mutliple Categories with the same name: "+needEntry.getKey());
+				System.out.println("Mutliple Categories with the same name: " + categoryName);
 			}
 		}
 		return result;
@@ -250,8 +256,7 @@ public class RuntimeUtil {
 			}
 		}
 		ticks = ticks*scaleFactor;
-		Time result = getExecutionTimeForCycles(ticks, processingUnit.getFrequencyDomain().getDefaultValue());
-		return result;
+		return getExecutionTimeForCycles(ticks, processingUnit.getFrequencyDomain().getDefaultValue());
 	}
 
 	/**
@@ -271,11 +276,35 @@ public class RuntimeUtil {
 		Time result = t1.multiply(factor);
 		return result.adjustUnit();
 	}
-
-
-	//TODO: Time to ticks?
 	
+	/**
+	 * Computes ticks for a given time (in seconds) and frequency.
+	 * 
+	 * @param executionTime execution time in seconds
+	 * @param frequency     frequency (of an executing processing unit)
+	 * @return ticks / cycles
+	 */
+	public static double getTicksForExecutionTimeInSeconds(BigDecimal executionTime, Frequency frequency) {
+		if (executionTime == null || frequency == null) return Double.NaN;
+		
+		BigDecimal cyclesPerSecond = AmaltheaServices.convertToHertz(frequency);
+		BigDecimal ticks = executionTime.multiply(cyclesPerSecond);
+		
+		return ticks.doubleValue();
+	}
 	
+	/**
+	 * Computes ticks for a given time and frequency.
+	 * 
+	 * @param executionTime execution time
+	 * @param frequency     frequency (of an executing processing unit)
+	 * @return ticks / cycles
+	 */
+	public static double getTicksForExecutionTime(Time executionTime, Frequency frequency) {
+		if (executionTime == null) return Double.NaN;
+		return getTicksForExecutionTimeInSeconds(AmaltheaServices.convertToSeconds(executionTime), frequency);
+	}
+
 	/**
 	 * Sets the runtime of the given runnable to the given ExecutionNeed
 	 */
@@ -283,7 +312,7 @@ public class RuntimeUtil {
 		clearRuntimeOfRunnable(runnable,  modes);
 		addRuntimeToRunnable(runnable, need);
 	}
-	
+
 	/**
 	 * Sets the runtime of the given runnable to the given Ticks
 	 */
@@ -291,28 +320,23 @@ public class RuntimeUtil {
 		clearRuntimeOfRunnable(runnable,  modes);
 		addRuntimeToRunnable(runnable, ticks);
 	}
-	
-	
+
 	/**
-	 * Adds Runtime (given as ExecutionNeed) to an existing Runnable currently
-	 * without consideration of modes
+	 * Adds Runtime (given as ExecutionNeed) to an existing Runnable
+	 * currently without consideration of modes
 	 */
-	//TODO: Mode-based API
 	public static void addRuntimeToRunnable(Runnable runnable, ExecutionNeed need) {
 		runnable.getRunnableItems().add(need);
 	}
-	
-	
+
 	/**
-	 * Adds Runtime (given as Ticks) to an existing Runnable currently
-	 * without consideration of modes
+	 * Adds Runtime (given as Ticks) to an existing Runnable
+	 * currently without consideration of modes
 	 */
-	//TODO: Mode-based API
 	public static void addRuntimeToRunnable(Runnable runnable, Ticks ticks) {
 		runnable.getRunnableItems().add(ticks);
 	}
-	
-	
+
 	/**
 	 * Clears all runtime information (execution needs and ticks)
 	 * 
@@ -328,8 +352,7 @@ public class RuntimeUtil {
 			clearRuntimeOfProcess(process, modes);
 		}
 	}
-	
-	
+
 	/**
 	 * Clears all runtime information (execution needs and ticks)
 	 * 
@@ -339,11 +362,11 @@ public class RuntimeUtil {
 	public static void clearRuntimeOfProcess(Process process, EMap<ModeLabel, String> modes) {
 		List<ExecutionNeed> executionNeeds = SoftwareUtil.getExecutionNeeds(process, modes);
 		AmaltheaIndex.deleteAll(executionNeeds);
-		
+
 		List<Ticks> ticks = SoftwareUtil.getTicks(process, modes);
 		AmaltheaIndex.deleteAll(ticks);
 	}
-	
+
 	/**
 	 * Clears all runtime information (execution needs and ticks)
 	 * 
@@ -409,11 +432,11 @@ public class RuntimeUtil {
 
 		return run;
 	}		
-	
-	
+
+
 	//#######################################################
 
-	
+
 	/**
 	 * Calculates the utilization for a given procUnit
 	 */
@@ -437,7 +460,7 @@ public class RuntimeUtil {
 			List<HwFeature> hwFeatures, EMap<ModeLabel, String> modes) {
 		HashMap<ProcessingUnit, Double> utilizations = new HashMap<>();
 
-		Set<ProcessingUnit> procUnits = DeploymentUtil.getAssignedCoreForProcess(process, model);
+		Set<ProcessingUnit> procUnits = DeploymentUtil.getAssignedCoreForProcess(process);
 
 		for (ProcessingUnit procUnit : procUnits) {
 			double utilization = getProcessUtilization(process, procUnit, model, tt, hwFeatures, modes);
@@ -464,8 +487,7 @@ public class RuntimeUtil {
 			System.err.println("execTime " + time);
 		}
 		for (Time period : periods) {
-			// System.out.println(process.getName() + " Period:
-			// "+TimeUtil.timeToString(period) + " ET: "+TimeUtil.timeToString(time));
+			// System.out.println(process.getName() + " Period: " + TimeUtil.timeToString(period) + " ET: "+TimeUtil.timeToString(time));
 			if (period != null && period.getValue().intValue() != 0) {
 				utilization += time.divide(period);
 			}
@@ -539,39 +561,6 @@ public class RuntimeUtil {
 			if (stimulus instanceof PeriodicStimulus) {
 				PeriodicStimulus p = ((PeriodicStimulus) stimulus);
 				result.add(p.getRecurrence());
-				} else if (stimulus instanceof ArrivalCurveStimulus) {			//Simon 11.01.2019: TODO
-					switch (tt) {
-					case ACET:
-						break;
-					case BCET:
-						break;
-					case WCET:
-						break;
-					default:
-						break;
-					}
-			} else if (stimulus instanceof CustomStimulus) {
-				switch (tt) {
-				case ACET:
-					break;
-				case BCET:
-					break;
-				case WCET:
-					break;
-				default:
-					break;
-				}
-			} else if (stimulus instanceof EventStimulus) {
-				switch (tt) {
-				case ACET:
-					break;
-				case BCET:
-					break;
-				case WCET:
-					break;
-				default:
-					break;
-				}
 			} else if (stimulus instanceof InterProcessStimulus) {
 				InterProcessStimulus ip = ((InterProcessStimulus) stimulus);
 				long ipPrescaler = 1L;
@@ -585,13 +574,14 @@ public class RuntimeUtil {
 					triggeringProcesses.remove(process);
 				}
 
-				for (Process triggeringProcess : triggeringProcesses.keySet()) {
-					// System.out.println(" --triggered by "+triggeringProcess.getName());
-					// InterProcessActivation ipa = getIpaForStimulus(stimulus, triggeringProcess,
-					// modes);
+				for (Entry<Process, Long> entry : triggeringProcesses.entrySet()) {
+					Process triggeringProcess = entry.getKey();
+					// System.out.println(" --triggered by " + triggeringProcess.getName());
+					// InterProcessActivation ipa = getIpaForStimulus(stimulus, triggeringProcess, modes);
+					long ipaPrescaler = entry.getValue();
+					
 					List<Time> periods = getPeriodsOfProcess(triggeringProcess, tt, modes);
 					for (Time t : periods) {
-						long ipaPrescaler = triggeringProcesses.get(triggeringProcess);
 						// if(ipa.getCounter() != null) {
 						// ipaPrescaler = ipa.getCounter().getPrescaler();
 						// }
@@ -604,50 +594,6 @@ public class RuntimeUtil {
 				// System.out.println(getTriggeringProcess(model, ip));
 				// result.addAll(getPeriods(model, getTriggeringProcess(model, ip), tt));
 
-			} else if (stimulus instanceof VariableRateStimulus) {							//Simon 11.01.2019: TODO
-				VariableRateStimulus vr = ((VariableRateStimulus) stimulus);
-				// System.out.println(s);
-				Time time = null;
-				switch (tt) {
-				case ACET:
-					// TODO fix it
-					// if(vr.getStimulusDeviation() != null) {
-					// time = getMean(vr.getStimulusDeviation());
-					// }
-					// break;
-					// case BCET:
-					// if(vr.getStimulusDeviation() != null) {
-					// time = vr.getStimulusDeviation().getUpperBound();
-					// }
-					// break;
-					// case WCET:
-					// if(vr.getStimulusDeviation() != null) {
-					// time = vr.getStimulusDeviation().getLowerBound();
-					// }
-					break;
-				default:
-					break;
-				}
-
-				if (time == null) {
-					/*
-					 * time = AmaltheaFactory.eINSTANCE.createTime(); time.setValue(0);
-					 * time.setUnit(TimeUnit.MS);
-					 */
-				} else {
-					result.add(time);
-				}
-			} else if (stimulus instanceof SingleStimulus) {
-				switch (tt) {
-				case ACET:
-					break;
-				case BCET:
-					break;
-				case WCET:
-					break;
-				default:
-					break;
-				}
 			} else if (stimulus instanceof RelativePeriodicStimulus) {
 				RelativePeriodicStimulus rp = ((RelativePeriodicStimulus) stimulus);
 				Time time = null;
@@ -672,25 +618,9 @@ public class RuntimeUtil {
 					break;
 				}
 
-				if (time == null) {
-					/*
-					 * time = AmaltheaFactory.eINSTANCE.createTime(); time.setValue(0);
-					 * time.setUnit(TimeUnit.MS);
-					 */
-				} else {
+				if (time != null) {
 					result.add(time);
 				}
-			} else if (stimulus instanceof PeriodicSyntheticStimulus) {							//Simon 11.01.2019: TODO
-				switch (tt) {
-				case ACET:
-					break;
-				case BCET:
-					break;
-				case WCET:
-					break;
-				default:
-					break;
-				}
 			}
 		}
 
@@ -752,9 +682,10 @@ public class RuntimeUtil {
 
 		// TODO check
 		Map<Process, List<Time>> result = processes.stream().collect(Collectors.toMap(p -> p,
-				p -> p.getStimuli().stream().filter(s -> (s instanceof RelativePeriodicStimulus)).map(
-						s -> getActivationTimeFromDeviation(((RelativePeriodicStimulus) s).getNextOccurrence(), tt))
-						.collect(Collectors.toList())));
+				p -> p.getStimuli().stream()
+					.filter(RelativePeriodicStimulus.class::isInstance)
+					.map(s -> getActivationTimeFromDeviation(((RelativePeriodicStimulus) s).getNextOccurrence(), tt))
+					.collect(Collectors.toList())));
 
 		return result;
 	}
@@ -785,11 +716,12 @@ public class RuntimeUtil {
 		// TODO check
 		Map<Process, List<ITimeDeviation>> result = processes.stream()
 				.filter(p -> p.getStimuli().stream()
-						.filter(s -> (s instanceof RelativePeriodicStimulus))
+						.filter(RelativePeriodicStimulus.class::isInstance)
 						.collect(Collectors.toList())
 						.size() > 0)
 				.collect(Collectors.toMap(p -> p,
-						p -> p.getStimuli().stream().filter(s -> (s instanceof RelativePeriodicStimulus))
+						p -> p.getStimuli().stream()
+								.filter(RelativePeriodicStimulus.class::isInstance)
 								.map(s -> ((RelativePeriodicStimulus) s).getNextOccurrence())
 								.collect(Collectors.toList())));
 
@@ -825,7 +757,7 @@ public class RuntimeUtil {
 
 		for (Stimulus stimulus : process.getStimuli()) {
 
-			if ((filter == null) || (filter.apply(stimulus))) {
+			if (filter == null || Boolean.TRUE.equals(filter.apply(stimulus)) ) {
 
 				if (stimulus instanceof PeriodicStimulus) {
 					PeriodicStimulus p = ((PeriodicStimulus) stimulus);
@@ -846,11 +778,12 @@ public class RuntimeUtil {
 						ipPrescaler = ip.getCounter().getPrescaler();
 					}
 					Map<Process, Long> triggeringProcesses = getTriggeringProcesses(ip, null);
-					for (Process triggeringProcess : triggeringProcesses.keySet()) {
+					for (Entry<Process, Long> entry : triggeringProcesses.entrySet()) {
+						Process triggeringProcess = entry.getKey();
 						if (triggeringProcess.equals(process)) {
 							continue; // don't follow loops
 						}
-						long ipaPrescaler = triggeringProcesses.get(triggeringProcess);
+						long ipaPrescaler = entry.getValue();
 						Map<Stimulus, Long> plainTriggerForProcess = getPlainTriggerForProcess(triggeringProcess, depthCounter * ipaPrescaler * ipPrescaler,
 								filter);
 						map.putAll(plainTriggerForProcess);
@@ -876,4 +809,10 @@ public class RuntimeUtil {
 		return map;
 	}
 
+	private static Amalthea getContainingModel(EObject object) {
+		if (object == null) return null;
+		
+		return AmaltheaServices.getContainerOfType(object, Amalthea.class);
+	}
+
 }
